import curses


# from onboarding_project_ricardo_caetano.game_logic import GAME_TITLE, GAME_INSTRUCTIONS, Y_STEP, X_STEP, OCCUPIED_POSITION, DRAW_GAME, Y_OFFSET, \
#     P1_MARKER, P2_MARKER, MARK_WAS_PLACED, P1_WON, P2_WON
# from onboarding_project_ricardo_caetano.game_sounds import player1_sound, player2_sound, victory_sound, tie_sound
from game_logic.game import Game
from game_logic.game_settings import MARK_WAS_PLACED, OCCUPIED_POSITION, DRAW_GAME, P1_WON, P2_WON, GAME_INSTRUCTIONS, GAME_TITLE, \
    P1_MARKER, Y_STEP, X_STEP, Y_OFFSET, P2_MARKER
from .baseUI import TextUI
from game_sounds.game_sounds import player1_sound, player2_sound, tie_sound, victory_sound

TIE_FIGHTER = """
          ._,.
           "..-..pf.
          -L   ..#'
        .+_L  ."]#
        ,'j' .+.j`                 -'.__..,.,p.
       _~ #..<..0.                 .J-.``..._f.
      .7..#_.. _f.                .....-..,`4'
      ;` ,#j.  T'      ..         ..J....,'.j`
     .` .."^.,-0.,,,,yMMMMM,.    ,-.J...+`.j@
    .'.`...' .yMMMMM0M@^=`""g.. .'..J..".'.jH
    j' .'1`  q'^)@@#"^".`"='BNg_...,]_)'...0-
   .T ...I. j"    .'..+,_.'3#MMM0MggCBf....F.
   j/.+'.{..+       `^~'-^~~'''''''''''``'''
   ..... .y.}                  `.._-:`_...jf
   g-.  .Lg'                 ..,..'-....,'.
  .'.   .Y^                  .....',].._f
  ......-f.                 .-,,.,.-:--&`
                            .`...'..`_J`
                            .~......'#'
                             '..,,.,_]`     
                            .L..`..``.  
"""


class Render(TextUI):
    def __init__(self):
        super().__init__()
        self.window_lines = 0
        self.window_cols = 0
        self.game = Game()

    def input(self, key):
        """
            Handles player inputs.

            Runs the game depending on pressed key.

            :param key: String representation of pressed key.

            :return: nothing
        """

        # Quit the game anytime
        if key in ["q", "Q"] or key and not self.game.game_is_on:
            self.stop()

        # Move cursor or place marker accordingly to input
        play_output = self.game.player_input(key)

        self.screen.clear()

        if play_output == MARK_WAS_PLACED:
            player1_sound() if self.game.is_player1_turn else player2_sound()

        if play_output == OCCUPIED_POSITION:
            self.display_occupied_position_message()
            curses.beep()
        if play_output == DRAW_GAME:
            self.screen.clear()
            tie_sound()
            self.display_tie_message(play_output)
            self.end_game()

        if play_output == P1_WON or play_output == P2_WON:
            self.screen.clear()
            victory_sound()
            self.display_winning_message(play_output)
            self.end_game()

    def end_game(self):
        self.game.game_is_on = False
        curses.curs_set(0)
        self.draw()
        self.pause()

    def display_occupied_position_message(self):
        self.screen.addstr(30, 20, str("Position already taken."))

    def display_tie_message(self, play_output):
        self.screen.addstr(2, 20, TIE_FIGHTER)

    def display_winning_message(self, play_output):
        self.screen.addstr(30, 20, str(play_output))

    def draw(self):
        """
        Handles the rendering of all game components into the screen.

        It's called continuously. Check TextUI class main loop.

        :return: Nothing
        """

        # self.screen.addstr(20, 20, str(self.game.board_state))
        # self.screen.addstr(22, 20, str(self.game.total_moves))
        # self.screen.addstr(24, 20, str(self.game.game_is_on))
        # self.screen.addstr(28, 20, str(self.game.position_x))
        # self.screen.addstr(30, 20, str(self.game.position_y))

        #        Nice border around the terminal
        self.screen.border()

        # How big is the terminal screen?
        self.window_lines = curses.LINES
        self.window_cols = curses.COLS

        # Where we will draw the board and title
        top_y = int(self.window_lines / 4)
        middle_x = int(self.window_cols / 2)

        # Adding title and instructions
        self.screen.addstr(1, middle_x - int(len(GAME_TITLE) / 2), GAME_TITLE)
        self.screen.addstr(3, middle_x - int(len(GAME_INSTRUCTIONS) / 2), GAME_INSTRUCTIONS)

        # Add the board.
        self.screen.addstr(top_y, middle_x,
                           f'{self.game.board_state[0][0]} │ {self.game.board_state[0][1]} │ {self.game.board_state[0][2]} ')
        self.screen.addstr(top_y + 1, middle_x, '──┼───┼──')
        self.screen.addstr(top_y + 2, middle_x,
                           f'{self.game.board_state[1][0]} │ {self.game.board_state[1][1]} │ {self.game.board_state[1][2]} ')
        self.screen.addstr(top_y + 3, middle_x, '──┼───┼──')
        self.screen.addstr(top_y + 4, middle_x,
                           f'{self.game.board_state[2][0]} │ {self.game.board_state[2][1]} │ {self.game.board_state[2][2]} ')

        # Player info including whose turn is it.
        self.show_players_info()

        # Move cursor to new position
        self.screen.move(top_y + self.game.position_y * Y_STEP, middle_x + self.game.position_x * X_STEP)

    def show_players_info(self):
        """
        Show whose turn it is, only when the game is on.

        :return: Nothing
        """
        if self.game.game_is_on:
            self.screen.addstr(Y_OFFSET + 6, 1, f'Player {P1_MARKER}',
                               curses.A_BLINK if self.game.is_player1_turn else 0)
            self.screen.addstr(Y_OFFSET + 7, 1, f'Player {P2_MARKER}',
                               curses.A_BLINK if not self.game.is_player1_turn else 0)
