import os
import time

NOTES_FREQUENCIES = {
    "C4": 261.63,
    "D4": 293.66,
    "D4#": 311.13,
    "E4": 329.63,
    "F4": 349.23,
    "G4": 392,
    "A4": 440,
    "B4": 493.88,
    "C5": 523.25
}


def beep(note, duration):
    """
    Plays a sound of the given 'note' with a given 'duration'

    :param note: (string) A representation of a musical note. Check NOTES_FREQUENCIES for available notes.
    :param duration: (int) The duration expressed in milliseconds.

    :return: Nothing
    """
    frequency = get_frequency(note)
    os.system('play --no-show-progress -n -t alsa synth %s sin %s' % (duration / 1000, frequency))


def get_frequency(note):
    """
    Returns the frequency in Hz of the given note. Check NOTES_FREQUENCIES for available notes.

    :param note: (string) A representation of a musical note. Check NOTES_FREQUENCIES for available notes.

    :return: (float) Frequency of the given 'note'
    """
    frequency = NOTES_FREQUENCIES[note]
    return frequency


def player1_sound():
    """
    Player 1 marking sound
    :return: Nothing
    """
    beep("G4", 100)
    beep("C5", 50)


def player2_sound():
    """
    Player 2 marking sound
    :return: Nothing
    """
    beep("C5", 50)
    beep("G4", 100)


def play_intro():
    """
    Game intro sound

    :return: Nothing
    """
    beep("G4", 120)
    beep("G4", 120)
    beep("G4", 120)
    beep("D4#", 600)
    time.sleep(0.3)
    beep("F4", 120)
    beep("F4", 120)
    beep("F4", 120)
    beep("D4", 600)


def victory_sound():
    """
      Victory sound

      :return: Nothing
    """
    beep("G4", 80)
    beep("G4", 80)
    beep("G4", 80)
    beep("C5", 160)
    beep("G4", 80)
    beep("C5", 300)


def tie_sound():
    """
      Tie sound

      :return: Nothing
    """
    time.sleep(0.3)
    beep("C5", 160)
    time.sleep(0.1)
    beep("C4", 80)
    beep("C4", 80)
